import { Utilities } from '@/util/Utilities';
import {
    DialogService,
    Form,
    LoadingService,
    Modal, ModalService, Select, SelectMultiple, Text, TextInput,
} from '@zeedhi/common';
import { Http, Metadata } from '@zeedhi/core';
import { TekGrid } from '@zeedhi/teknisa-components-common';
import { RichText } from '@zeedhi/zd-richtext';

export class Sm00005000TabParametrizacoesServicesEmailController {
    private basicModal!: Modal;

    private clicks!: any;

    public currentRow: any = {
        NRPARAMEMAILFILIAL: null,
    };

    // função que simula um double click
    public rowClick({ row }: any) {
        this.currentRow = row;
        this.clicks += 1;
        if (this.clicks >= 2) {
            this.abrirModalEmailEdicao();
            this.clicks = 0;
        }
        setTimeout(() => {
            this.clicks = 0;
        }, 500);
    }

    public async criarModalEmail() {
        this.basicModal = await ModalService.createFromJson('sm_00005_000_parametrizacoes_services/modal/sm_00005_000_parametrizacoes_services_emails_modal', true);
    }
    
    public async exibirModalEmail() {
        this.basicModal.show();
    }

    public async fecharModalEmail() {
        await this.limparCamposModal();
        this.basicModal?.hide();
    }

    public destruirModal() {
        this.basicModal?.destroy();
    }

    public limparCamposModal() {
        const form = Metadata.getInstance<Form>('form_modal_email');
        const texto = Metadata.getInstance<Text>('text-Mnemonicos');
        const corpoEmail = Metadata.getInstance<RichText>('DSCORPOEMAIL');
        form.clearForm();
        texto.isVisible = false;
        corpoEmail.value = '';
    }

    public async abrirModalEmailInsert() {
        await this.exibirModalEmail();
        Metadata.getInstance<Text>('titleText-parametrizacaoEmail').text = 'Novo Registro';
        this.currentRow.NRPARAMEMAILFILIAL = null;
    }

    public async abrirModalEmailEdicao() {
        await this.exibirModalEmail();
        Metadata.getInstance<Text>('titleText-parametrizacaoEmail').text = 'Edição de Registro';
        Metadata.getInstance<TextInput>('DSPARAMEMAILFILIAL').value = this.currentRow.DSPARAMEMAILFILIAL;
        Metadata.getInstance<Select>('CDFILIAL').value = this.currentRow.CDFILIAL;
        Metadata.getInstance<TextInput>('DSREMETENTEEMAIL').value = this.currentRow.DSREMETENTEEMAIL;
        Metadata.getInstance<TextInput>('DSASSUNTOEMAIL').value = this.currentRow.DSASSUNTOEMAIL;
        Metadata.getInstance<RichText>('DSCORPOEMAIL').value = this.currentRow.DSCORPOEMAIL;
        Metadata.getInstance<SelectMultiple>('NRTIPOORCAMENTO').setValue(Utilities.formatarStringParaArray(this.currentRow.NRTIPOORCAMENTO));
    }

    public changeEmail({ component }: any) {
        const result = this.validarEmail(component.value);
        if (result === false) {
            Utilities.showDialogMessage('error', 'Operação Bloqueada!', 'E-mail inválido');
        }
    }

    public validarEmail(email: any) {
        const re = /\S+@\S+\.\S+/;
        return re.test(email);
    }

    public saveParameterEmail() {
        const form = Metadata.getInstance<Form>('form_modal_email');
        const grid = Metadata.getInstance<TekGrid>('basic-tek-grid-tab-emails');

        if (Utilities.validateForm(form)) {
            LoadingService.show();

            const nrparamemailfilial = this.currentRow.NRPARAMEMAILFILIAL ? this.currentRow.NRPARAMEMAILFILIAL : null;

            Http.post('/saveParameterEmail', {
                CDFILIAL: Metadata.getInstance<Select>('CDFILIAL').value,
                DSPARAMEMAILFILIAL: Metadata.getInstance<TextInput>('DSPARAMEMAILFILIAL').value,
                DSREMETENTEEMAIL: Metadata.getInstance<TextInput>('DSREMETENTEEMAIL').value,
                DSASSUNTOEMAIL: Metadata.getInstance<TextInput>('DSASSUNTOEMAIL').value,
                DSCORPOEMAIL: Metadata.getInstance<RichText>('DSCORPOEMAIL').value,
                NRTIPOORCAMENTO: Metadata.getInstance<SelectMultiple>('NRTIPOORCAMENTO').value,
                NRPARAMEMAILFILIAL: nrparamemailfilial,
            }).then(() => {
                this.fecharModalEmail();
                grid.datasource.get();
                Utilities.showSuccessMessage();
            }).catch((error) => {
                if (error.response.data.error === 'EXISTE') {
                    Utilities.showDialogMessage('error', 'Operação bloqueada!', 'Filial selecionada já possui e-mail parametrizado.');
                } else if (error.response.data.error === 'CONTEM_ESCOPO') {
                    Utilities.showDialogMessage('error', 'Operação bloqueada!', 'Já existe um registro associado ao(s) escopo(s) selecionado(s).');
                } else {
                    Utilities.showErrorMessage(error);
                }
            }).finally(() => {
                LoadingService.hide();
            });
        }
    }

    public showConfirmDeleteMessage({ event }: any) {
        event!.preventDefault();
        DialogService.show({
            name: 'buttons-dialog-operador',
            type: 'warning',
            title: 'Excluir Registro',
            text: 'Deseja realmente excluir todos os registros selecionados?',
            buttons: [
                {
                    name: 'NAO',
                    component: 'ZdButton',
                    label: 'Não',
                    outline: true,
                    events: {
                        click: () => {
                            DialogService.hide();
                        },
                    },
                },
                {
                    name: 'SIM',
                    component: 'ZdButton',
                    label: 'Sim',
                    events: {
                        click: () => {
                            this.deleteParametrosEmail();
                            DialogService.hide();
                        },
                    },
                },
            ],
        });
    }

    public deleteParametrosEmail() {
        const grid = Metadata.getInstance<TekGrid>('basic-tek-grid-tab-emails');
        LoadingService.show();
        Http.post('/deleteParametrosEmail', grid.selectedRows)
            .then(() => {
                grid.datasource.get();
                Utilities.showSuccessMessage();
            }).catch((error) => {
                Utilities.showErrorMessage(error);
            }).finally(() => {
                LoadingService.hide();
            });
    }

    public visibilidadeTextoMnemonicos() {
        const texto = Metadata.getInstance<Text>('text-Mnemonicos');
        texto.isVisible = !texto.isVisible;
    }
}
